/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is gContactSync.
 *
 * The Initial Developer of the Original Code is
 * Josh Geenen <joshgeenen+contactphotos@gmail.com>.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

if (!com) {
  /** A generic wrapper variable */
  var com = {};
}

if (!com.ContactPhotos) {
  /** A wrapper for all CP functions and variables */
  com.ContactPhotos = {};
}

window.addEventListener("load",
  /**
   * Calls com.ContactPhotos.init when the window has loaded.
   */
  function CP_loadListener(e) {
    com.ContactPhotos.init();
  },
false);

/**
 * Initializes this add-on.
 * This method inserts an hbox in between the expandedHeaderView vbox and
 * the expandedHeadersBox vbox in order to add an html:img element to the left
 * of the original header information.  The img contains the photo of the
 * contact who sent the message.  If the sender is not a contact, or if the
 * contact does not have a photo, the generic photo is used.
 */
com.ContactPhotos.init = function ContactPhotos_init(aURI) {
  let viewBox    = document.getElementById("expandedHeaderView");
  var newHbox    = document.createElement("hbox");
  // TODO: why don't hbox and vbox work with maxwidth?
  let imgBox     = document.createElement("description");
  let img        = document.createElementNS("http://www.w3.org/1999/xhtml",
                                           "html:img");
  var children   = [];
  let maxWidth   = com.ContactPhotos.Preferences.mMaxWidth;
  let maxHeight  = com.ContactPhotos.Preferences.mMaxHeight;
  
  // remove all children from viewBox
  for (let i = 0; i < viewBox.childNodes.length; i++) {
    children.push(viewBox.removeChild(viewBox.childNodes[i]));
  }

  // setup the max height and width
  imgBox.style.maxWidth  = img.style.maxWidth  = maxWidth;
  imgBox.style.maxHeight = img.style.maxHeight = maxHeight;
  img.setAttribute("valign", "middle");
  imgBox.setAttribute("valign", "middle");
  imgBox.style.textAlign = "center";
  // give the img an ID
  img.setAttribute("id",    "msgHdrFromPhoto");
  imgBox.setAttribute("id", "msgHdrFromPhotoBox");
  // set the default src of the image to the default photo URI
  img.setAttribute("src", defaultPhotoURI);
  // add the html:img to the imgBox
  imgBox.appendChild(img);
  if (com.ContactPhotos.Preferences.mDisplayOnLeft) {
    // add imgBox to the new hbox (first so it is on the left)
    newHbox.appendChild(imgBox);
  }
  // add the old children of expandedHeaderView (expandedHeadersBox) to the hbox
  while (children.length > 0) {
    newHbox.appendChild(children.shift());
  }
  if (!com.ContactPhotos.Preferences.mDisplayOnLeft) {
    // add imgBox to the new hbox (last so it is on the right)
    newHbox.appendChild(imgBox);
  }
  // add the new hbox to expandedHeaderView
  viewBox.appendChild(newHbox);

  com.ContactPhotos.originalAddExtraAddressProcessing = AddExtraAddressProcessing;
  AddExtraAddressProcessing = com.ContactPhotos.AddExtraAddressProcessing;
};

/** The original AddExtraAddressProcessing function */
com.ContactPhotos.originalAddExtraAddressProcessing = function() {};

/**
 * Meant to override AddExtraAddressProcessing.
 * This function calls com.ContactPhotos.originalAddressExtraAddressProcessing
 * and then updates the src of the photo with the contact's photo, if any, and
 * defaults to the defaultPhotoURI set in abCommon.js
 */
com.ContactPhotos.AddExtraAddressProcessing = function CP_extraAdrProcessing(aEmail, aNode) {
  // call the original
  com.ContactPhotos.originalAddExtraAddressProcessing.apply(this, arguments);
  // if aNode is part of the From box, then use it for the photo
  let id = aNode.parentNode.parentNode.parentNode.id;
  if (id === "expandedfromBox") {
    let photoElem = document.getElementById("msgHdrFromPhoto");
    let photoURI  = defaultPhotoURI;
    if (aNode.getAttribute("hascard") === "true") {
      let card  = aNode.cardDetails.card;
      photoURI  = getPhotoURI(card.getProperty("PhotoName", null));
    }
    photoElem.setAttribute("src", photoURI);
  }
};
